@@ -11,10 +11,14 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "debuggerpart.h"
+#include "label_with_double_click.h"
+
 #include <qdir.h>
 #include <qvbox.h>
 #include <qwhatsthis.h>
 #include <qpopupmenu.h>
+#include <qtooltip.h>
 
 #include <kaction.h>
 #include <kdebug.h>
@@ -31,6 +35,7 @@
 #include <dcopclient.h>
 #include <qtimer.h>
 #include <kstringhandler.h>
+#include <kdockwidget.h>
 
 #include "kdevcore.h"
 #include "kdevproject.h"
@@ -59,7 +64,9 @@
 #include <kdevplugininfo.h>
 #include <debugger.h>
 
-#include "debuggerpart.h"
+
+
+
 
 namespace GDBDebugger
 {
@@ -71,8 +78,9 @@ K_EXPORT_COMPONENT_FACTORY( libkdevdebugger, DebuggerFactory( data ) )
 
 DebuggerPart::DebuggerPart( QObject *parent, const char *name, const QStringList & ) :
     KDevPlugin( &data, parent, name ? name : "DebuggerPart" ),
-    controller(0), previousDebuggerState_(s_dbgNotStarted), 
-    justRestarted_(false)
+    controller(0), previousDebuggerState_(s_dbgNotStarted),
+    justRestarted_(false), needRebuild_(true),
+    running_(false)
 {
     setObjId("DebuggerInterface");
     setInstance(DebuggerFactory::instance());
@@ -80,18 +88,21 @@ DebuggerPart::DebuggerPart( QObject *parent, const char *name, const QStringList
     setXMLFile("kdevdebugger.rc");
 
     m_debugger = new Debugger( partController() );
-    
-    statusBarIndicator = new QLabel(" ", mainWindow()->statusBar());
+
+    statusBarIndicator = new LabelWithDoubleClick(
+        " ", mainWindow()->statusBar());
     statusBarIndicator->setFixedWidth(15);
+    statusBarIndicator->setAlignment(Qt::AlignCenter);
     mainWindow()->statusBar()->addWidget(statusBarIndicator, 0, true);
     statusBarIndicator->show();
 
     // Setup widgets and dbgcontroller
-    variableWidget = new VariableWidget( 0, "variablewidget");
-    mainWindow()->embedSelectView(variableWidget, i18n("Variables"), i18n("Debugger variable-view"));
+
+    controller = new GDBController(*projectDom());
 
 
-    gdbBreakpointWidget = new GDBBreakpointWidget( 0, "gdbBreakpointWidget" );
+    gdbBreakpointWidget = new GDBBreakpointWidget( controller,
+                                                   0, "gdbBreakpointWidget" );
     gdbBreakpointWidget->setCaption(i18n("Breakpoint List"));
     QWhatsThis::add
         (gdbBreakpointWidget, i18n("<b>Breakpoint list</b><p>"
@@ -103,7 +114,14 @@ DebuggerPart::DebuggerPart( QObject *parent, const char *name, const QStringList
     gdbBreakpointWidget->setIcon( SmallIcon("stop") );
     mainWindow()->embedOutputView(gdbBreakpointWidget, i18n("Breakpoints"), i18n("Debugger breakpoints"));
 
-    framestackWidget = new FramestackWidget( 0, "framestackWidget" );
+    variableWidget = new VariableWidget( controller,
+                                         gdbBreakpointWidget,
+                                         0, "variablewidget");
+    mainWindow()->embedSelectView(variableWidget, i18n("Variables"),
+                                  i18n("Debugger variable-view"));
+    mainWindow()->setViewAvailable(variableWidget, false);
+
+    framestackWidget = new FramestackWidget( controller, 0, "framestackWidget" );
     framestackWidget->setEnabled(false);
     framestackWidget->setCaption(i18n("Frame Stack"));
     QWhatsThis::add
@@ -119,7 +137,7 @@ DebuggerPart::DebuggerPart( QObject *parent, const char *name, const QStringList
     mainWindow()->embedOutputView(framestackWidget, i18n("Frame Stack"), i18n("Debugger function call stack"));
     mainWindow()->setViewAvailable(framestackWidget, false);
 
-    disassembleWidget = new DisassembleWidget( 0, "disassembleWidget" );
+    disassembleWidget = new DisassembleWidget( controller, 0, "disassembleWidget" );
     disassembleWidget->setEnabled(false);
     disassembleWidget->setCaption(i18n("Machine Code Display"));
     QWhatsThis::add
@@ -147,12 +165,6 @@ DebuggerPart::DebuggerPart( QObject *parent, const char *name, const QStringList
                                   i18n("GDB output"));
     mainWindow()->setViewAvailable(gdbOutputWidget, false);
 
-    VariableTree *variableTree = variableWidget->varTree();
-
-    // variableTree -> framestackWidget
-    connect( variableTree,     SIGNAL(selectFrame(int, int)),
-             framestackWidget, SLOT(slotSelectFrame(int, int)));
-
     // gdbBreakpointWidget -> this
     connect( gdbBreakpointWidget, SIGNAL(refreshBPState(const Breakpoint&)),
              this,             SLOT(slotRefreshBPState(const Breakpoint&)));
@@ -161,6 +173,15 @@ DebuggerPart::DebuggerPart( QObject *parent, const char *name, const QStringList
     connect( gdbBreakpointWidget, SIGNAL(gotoSourcePosition(const QString&, int)),
              this,             SLOT(slotGotoSource(const QString&, int)) );
 
+
+    viewerWidget = new ViewerWidget( controller, 0, "viewerWidget");
+    mainWindow()->embedSelectView(viewerWidget,
+                                  i18n("Debug views"),
+                                  i18n("Special debugger views"));
+    mainWindow()->setViewAvailable(viewerWidget, false);
+    connect(viewerWidget, SIGNAL(setViewShown(bool)),
+            this, SLOT(slotShowView(bool)));
+
     // Now setup the actions
     KAction *action;
 
@@ -327,6 +348,21 @@ DebuggerPart::DebuggerPart( QObject *parent, const char *name, const QStringList
     connect( procLineMaker, SIGNAL(receivedStderrLine(const QString&)),
              appFrontend(), SLOT(insertStderrLine(const QString&)) );
 
+    // The output from tracepoints goes to "application" window, because
+    // we don't have any better alternative, and using yet another window
+    // is undesirable. Besides, this makes tracepoint look even more similar
+    // to printf debugging.
+    connect( gdbBreakpointWidget,   SIGNAL(tracingOutput(const char*)),
+             procLineMaker,         SLOT(slotReceivedStdout(const char*)));
+
+
+    connect(partController(), SIGNAL(savedFile(const KURL &)),
+            this, SLOT(slotFileSaved()));
+
+    if (project())
+        connect(project(), SIGNAL(projectCompiled()),
+                this, SLOT(slotProjectCompiled()));
+
     setupController();
     QTimer::singleShot(0, this, SLOT(setupDcop()));
 }
@@ -348,15 +384,7 @@ void DebuggerPart::slotDCOPApplicationRegistered(const QCString& appId)
         QByteArray answer;
         QCString replyType;
 
-#if defined(KDE_MAKE_VERSION)
-# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,90)
         kapp->dcopClient()->call(appId, "krashinfo", "appName()", QByteArray(), replyType, answer, true, 5000);
-# else
-        kapp->dcopClient()->call(appId, "krashinfo", "appName()", QByteArray(), replyType, answer, true);
-# endif
-#else
-        kapp->dcopClient()->call(appId, "krashinfo", "appName()", QByteArray(), replyType, answer, true);
-#endif
 
         QDataStream d(answer, IO_ReadOnly);
         QCString appName;
@@ -374,15 +402,7 @@ ASYNC DebuggerPart::slotDebugExternalProcess()
     QByteArray answer;
     QCString replyType;
 
-#if defined(KDE_MAKE_VERSION)
-# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,90)
     kapp->dcopClient()->call(kapp->dcopClient()->senderId(), "krashinfo", "pid()", QByteArray(), replyType, answer, true, 5000);
-# else
-    kapp->dcopClient()->call(kapp->dcopClient()->senderId(), "krashinfo", "pid()", QByteArray(), replyType, answer, true);
-# endif
-#else
-    kapp->dcopClient()->call(kapp->dcopClient()->senderId(), "krashinfo", "pid()", QByteArray(), replyType, answer, true);
-#endif
 
     QDataStream d(answer, IO_ReadOnly);
     int pid;
@@ -397,6 +417,11 @@ ASYNC DebuggerPart::slotDebugExternalProcess()
     mainWindow()->main()->raise();
 }
 
+ASYNC DebuggerPart::slotDebugCommandLine(const QString& /*command*/)
+{
+    KMessageBox::information(0, "Asked to debug command line");
+}
+
 void DebuggerPart::slotCloseDrKonqi()
 {
     kapp->dcopClient()->send(m_drkonqi, "MainApplication-Interface", "quit()", QByteArray());
@@ -450,7 +475,7 @@ void DebuggerPart::contextMenu(QPopupMenu *popup, const Context *context)
 
     bool running = !(previousDebuggerState_ & s_dbgNotStarted);
 
-    // If debugger is running, we insert items at the top. 
+    // If debugger is running, we insert items at the top.
     // The reason is user has explicitly run the debugger, so he's
     // surely debugging, not editing code or something. So, first
     // menu items should be about debugging, not some copy/paste/cut
@@ -458,7 +483,7 @@ void DebuggerPart::contextMenu(QPopupMenu *popup, const Context *context)
     if (!running)
         popup->insertSeparator();
 
-    int index = running ? 0 : -1;        
+    int index = running ? 0 : -1;
     if (running)
     {
         // Too bad we can't add QAction to popup menu in Qt3.
@@ -476,7 +501,7 @@ void DebuggerPart::contextMenu(QPopupMenu *popup, const Context *context)
     }
     if (econtext->url().isLocalFile())
     {
-        int id = popup->insertItem( i18n("Toggle Breakpoint"), 
+        int id = popup->insertItem( i18n("Toggle Breakpoint"),
                                     this, SLOT(toggleBreakpoint()),
                                     0, -1, index);
         index += running;
@@ -485,12 +510,12 @@ void DebuggerPart::contextMenu(QPopupMenu *popup, const Context *context)
     if (!m_contextIdent.isEmpty())
     {
         QString squeezed = KStringHandler::csqueeze(m_contextIdent, 30);
-        int id = popup->insertItem( i18n("Evaluate: %1").arg(squeezed), 
+        int id = popup->insertItem( i18n("Evaluate: %1").arg(squeezed),
                                     this, SLOT(contextEvaluate()),
                                     0, -1, index);
         index += running;
         popup->setWhatsThis(id, i18n("<b>Evaluate expression</b><p>Shows the value of the expression under the cursor."));
-        int id2 = popup->insertItem( i18n("Watch: %1").arg(squeezed), 
+        int id2 = popup->insertItem( i18n("Watch: %1").arg(squeezed),
                                      this, SLOT(contextWatch()),
                                     0, -1, index);
         index += running;
@@ -530,7 +555,7 @@ void DebuggerPart::contextEvaluate()
 
 void DebuggerPart::projectConfigWidget(KDialogBase *dlg)
 {
-	QVBox *vbox = dlg->addVBoxPage(i18n("Debugger"), i18n("Debugger"), BarIcon( info()->icon(), KIcon::SizeMedium) );
+    QVBox *vbox = dlg->addVBoxPage(i18n("Debugger"), i18n("Debugger"), BarIcon( info()->icon(), KIcon::SizeMedium) );
     DebuggerConfigWidget *w = new DebuggerConfigWidget(this, vbox, "debugger config widget");
     connect( dlg, SIGNAL(okClicked()), w, SLOT(accept()) );
     connect( dlg, SIGNAL(finished()), controller, SLOT(configure()) );
@@ -541,66 +566,22 @@ void DebuggerPart::setupController()
 {
     VariableTree *variableTree = variableWidget->varTree();
 
-    controller = new GDBController(variableTree, framestackWidget, *projectDom());
-
-    // variableTree -> controller
-    connect( variableTree,          SIGNAL(expandItem(TrimmableItem*)),
-             controller,            SLOT(slotExpandItem(TrimmableItem*)));
-    connect( variableTree,          SIGNAL(expandUserItem(VarItem*, const QCString&)),
-             controller,            SLOT(slotExpandUserItem(VarItem*, const QCString&)));
-    connect( variableTree,          SIGNAL(setLocalViewState(bool)),
-             controller,            SLOT(slotSetLocalViewState(bool)));
-    connect( variableTree,          SIGNAL(varItemConstructed(VarItem*)),
-             controller,            SLOT(slotVarItemConstructed(VarItem*)));     // jw
-    connect( variableTree,          SIGNAL(produceVariablesInfo()),
-             controller,            SLOT(slotProduceVariablesInfo()));
-    connect( variableTree,          SIGNAL(setValue(const QString&, 
-                                                    const QString&)),
-             controller,            SLOT(slotSetValue(const QString&, 
-                                                      const QString&)));
-
     // variableTree -> gdbBreakpointWidget
     connect( variableTree,          SIGNAL(toggleWatchpoint(const QString &)),
              gdbBreakpointWidget,   SLOT(slotToggleWatchpoint(const QString &)));
 
-    // framestackWidget -> controller
-    connect( framestackWidget,      SIGNAL(selectFrame(int,int,bool)),
-             controller,            SLOT(slotSelectFrame(int,int,bool)));
-    connect( framestackWidget,      SIGNAL(produceBacktrace(int)),
-             controller,            SLOT(slotProduceBacktrace(int)));
-
-
-    // gdbBreakpointWidget -> controller
-    connect( gdbBreakpointWidget,   SIGNAL(clearAllBreakpoints()),
-             controller,            SLOT(slotClearAllBreakpoints()));
-    connect( gdbBreakpointWidget,   SIGNAL(publishBPState(const Breakpoint&)),
-             controller,            SLOT(slotBPState(const Breakpoint &)));
-
-    // disassembleWidget -> controller
-    connect( disassembleWidget,     SIGNAL(disassemble(const QString&, const QString&)),
-             controller,            SLOT(slotDisassemble(const QString&, const QString&)));
-
     // gdbOutputWidget -> controller
     connect( gdbOutputWidget,       SIGNAL(userGDBCmd(const QString &)),
              controller,            SLOT(slotUserGDBCmd(const QString&)));
     connect( gdbOutputWidget,       SIGNAL(breakInto()),
              controller,            SLOT(slotBreakInto()));
 
-    // controller -> gdbBreakpointWidget
-    connect( controller,            SIGNAL(acceptPendingBPs()),
-             gdbBreakpointWidget,   SLOT(slotSetPendingBPs()));
-    connect( controller,            SIGNAL(unableToSetBPNow(int)),
-             gdbBreakpointWidget,   SLOT(slotUnableToSetBPNow(int)));
-    connect( controller,            SIGNAL(rawGDBBreakpointList (char*)),
-             gdbBreakpointWidget,   SLOT(slotParseGDBBrkptList(char*)));
-    connect( controller,            SIGNAL(rawGDBBreakpointSet(char*, int)),
-             gdbBreakpointWidget,   SLOT(slotParseGDBBreakpointSet(char*, int)));
+    connect( controller,            SIGNAL(breakpointHit(int)),
+             gdbBreakpointWidget,   SLOT(slotBreakpointHit(int)));
 
     // controller -> disassembleWidget
     connect( controller,            SIGNAL(showStepInSource(const QString&, int, const QString&)),
              disassembleWidget,     SLOT(slotShowStepInSource(const QString&, int, const QString&)));
-    connect( controller,            SIGNAL(rawGDBDisassemble(char*)),
-             disassembleWidget,     SLOT(slotDisassemble(char*)));
 
     // controller -> this
     connect( controller,            SIGNAL(dbgStatus(const QString&, int)),
@@ -608,41 +589,36 @@ void DebuggerPart::setupController()
     connect( controller,            SIGNAL(showStepInSource(const QString&, int, const QString&)),
              this,                  SLOT(slotShowStep(const QString&, int)));
     connect( controller,            SIGNAL(debuggerAbnormalExit()),
-	     this,                  SLOT(slotDebuggerAbnormalExit()));
+         this,                  SLOT(slotDebuggerAbnormalExit()));
+
+    connect(controller, SIGNAL(event(GDBController::event_t)),
+            this,       SLOT(slotEvent(GDBController::event_t)));
 
     // controller -> procLineMaker
     connect( controller,            SIGNAL(ttyStdout(const char*)),
              procLineMaker,         SLOT(slotReceivedStdout(const char*)));
-    // The output from tracepoints goes to "application" window, because
-    // we don't have any better alternative, and using yet another window
-    // is undesirable. Besides, this makes tracepoint look even more similar
-    // to printf debugging.
-    connect( controller,            SIGNAL(tracingOutput(const char*)),
-             procLineMaker,         SLOT(slotReceivedStdout(const char*)));
     connect( controller,            SIGNAL(ttyStderr(const char*)),
              procLineMaker,         SLOT(slotReceivedStderr(const char*)));
 
     // controller -> gdbOutputWidget
-    connect( controller,            SIGNAL(gdbStdout(const char*)),
-             gdbOutputWidget,       SLOT(slotReceivedStdout(const char*)) );
+    connect( controller,            SIGNAL(gdbInternalCommandStdout(const char*)),
+             gdbOutputWidget,       SLOT(slotInternalCommandStdout(const char*)) );
+    connect( controller,            SIGNAL(gdbUserCommandStdout(const char*)),
+             gdbOutputWidget,       SLOT(slotUserCommandStdout(const char*)) );
+
     connect( controller,            SIGNAL(gdbStderr(const char*)),
              gdbOutputWidget,       SLOT(slotReceivedStderr(const char*)) );
     connect( controller,            SIGNAL(dbgStatus(const QString&, int)),
              gdbOutputWidget,       SLOT(slotDbgStatus(const QString&, int)));
 
-    // controller -> variableTree
+    // controller -> viewerWidget
     connect( controller, SIGNAL(dbgStatus(const QString&, int)),
-             variableTree, SLOT(slotDbgStatus(const QString&, int)));
-    connect( controller, SIGNAL(parametersReady(const char*)),
-             variableTree, SLOT(slotParametersReady(const char*)));
-    connect( controller, SIGNAL(localsReady(const char*)),
-             variableTree, SLOT(slotLocalsReady(const char*)));
-    connect( controller, SIGNAL(currentFrame(int, int)),
-             variableTree, SLOT(slotCurrentFrame(int, int)));
+             viewerWidget, SLOT(slotDebuggerState(const QString&, int)));
+
+
+    connect(statusBarIndicator, SIGNAL(doubleClicked()),
+            controller, SLOT(explainDebuggerStatus()));
 
-    // gdbBreakpointWidget -> disassembleWidget
-    connect( gdbBreakpointWidget,   SIGNAL(publishBPState(const Breakpoint&)),
-             disassembleWidget,     SLOT(slotBPState(const Breakpoint &)));
 }
 
 
@@ -659,67 +635,81 @@ bool DebuggerPart::startDebugger()
         run_envvars   = project()->runEnvironmentVars();
         run_directory = project()->runDirectory();
         program       = project()->mainProgram();
-        run_arguments = DomUtil::readEntry(*projectDom(), "/kdevdebugger/general/programargs");
+        run_arguments = project()->debugArguments();
     }
 
     QString shell = DomUtil::readEntry(*projectDom(), "/kdevdebugger/general/dbgshell");
     if( !shell.isEmpty() )
     {
-        QFileInfo info( shell );
+        shell = shell.simplifyWhiteSpace();
+        QString shell_without_args = QStringList::split(QChar(' '), shell ).first();
+
+        QFileInfo info( shell_without_args );
         if( info.isRelative() )
         {
-            shell = build_dir + "/" + shell;
-            info.setFile( shell );
+            shell_without_args = build_dir + "/" + shell_without_args;
+            info.setFile( shell_without_args );
         }
         if( !info.exists() )
         {
-            KMessageBox::error(
+            KMessageBox::warningYesNo(
                 mainWindow()->main(),
-                i18n("Could not locate the debugging shell '%1'.").arg( shell ),
-                i18n("Debugging Shell Not Found") );
+                i18n("Could not locate the debugging shell '%1'.").arg( shell_without_args ),
+                i18n("Debugging Shell Not Found"),KStdGuiItem::ok(), KStdGuiItem::cont(), "gdb_error" );
             return false;
         }
     }
 
-    core()->running(this, true);
-
-    stateChanged( QString("active") );
-
-    KActionCollection *ac = actionCollection();
-    ac->action("debug_run")->setText( i18n("&Continue") );
-//    ac->action("debug_run")->setIcon( "dbgrun" );
-    ac->action("debug_run")->setToolTip( i18n("Continues the application execution") );
-    ac->action("debug_run")->setWhatsThis( i18n("Continue application execution\n\n"
-                                           "Continues the execution of your application in the "
-                                           "debugger. This only takes effect when the application "
-                                           "has been halted by the debugger (i.e. a breakpoint has "
-                                           "been activated or the interrupt was pressed).") );
-
-
-//    mainWindow()->setViewAvailable(variableWidget, true);
-    mainWindow()->setViewAvailable(framestackWidget, true);
-    mainWindow()->setViewAvailable(disassembleWidget, true);
-    mainWindow()->setViewAvailable(gdbOutputWidget, true);
-
-//     variableWidget->setEnabled(true);
-    framestackWidget->setEnabled(true);
-    disassembleWidget->setEnabled(true);
-
-    gdbOutputWidget->clear();
-    gdbOutputWidget->setEnabled(true);
-
-    if (DomUtil::readBoolEntry(*projectDom(), "/kdevdebugger/general/floatingtoolbar", false))
+    if (controller->start(shell, run_envvars, run_directory,
+                          program, run_arguments))
     {
-        floatingToolBar = new DbgToolBar(this, mainWindow()->main());
-        floatingToolBar->show();
-    }
+        core()->running(this, true);
 
-    controller->slotStart(shell, run_envvars, run_directory, program, run_arguments);
-    return true;
+        stateChanged( QString("active") );
+
+        KActionCollection *ac = actionCollection();
+        ac->action("debug_run")->setText( i18n("&Continue") );
+
+        ac->action("debug_run")->setToolTip(
+            i18n("Continues the application execution") );
+        ac->action("debug_run")->setWhatsThis(
+            i18n("Continue application execution\n\n"
+                 "Continues the execution of your application in the "
+                 "debugger. This only takes effect when the application "
+                 "has been halted by the debugger (i.e. a breakpoint has "
+                 "been activated or the interrupt was pressed).") );
+
+        mainWindow()->setViewAvailable(framestackWidget, true);
+        mainWindow()->setViewAvailable(disassembleWidget, true);
+        mainWindow()->setViewAvailable(gdbOutputWidget, true);
+        mainWindow()->setViewAvailable(variableWidget, true);
+
+        framestackWidget->setEnabled(true);
+        disassembleWidget->setEnabled(true);
+
+        gdbOutputWidget->setEnabled(true);
+
+
+        if (DomUtil::readBoolEntry(*projectDom(), "/kdevdebugger/general/floatingtoolbar", false))
+        {
+#ifndef QT_MAC
+            floatingToolBar = new DbgToolBar(this, mainWindow()->main());
+            floatingToolBar->show();
+#endif
+        }
+
+        running_ = true;
+        return true;
+    }
+    else
+    {
+        return false;
+    }
 }
 
 void DebuggerPart::slotStopDebugger()
 {
+    running_ = false;
     controller->slotStopDebugger();
     debugger()->clearExecutionPoint();
 
@@ -727,9 +717,8 @@ void DebuggerPart::slotStopDebugger()
     floatingToolBar = 0;
 
     gdbBreakpointWidget->reset();
-    framestackWidget->clear();
-    variableWidget->clear();
     disassembleWidget->clear();
+    gdbOutputWidget->clear();
     disassembleWidget->slotActivate(false);
 
 //     variableWidget->setEnabled(false);
@@ -737,7 +726,8 @@ void DebuggerPart::slotStopDebugger()
     disassembleWidget->setEnabled(false);
     gdbOutputWidget->setEnabled(false);
 
-//    mainWindow()->setViewAvailable(variableWidget, false);
+
+    mainWindow()->setViewAvailable(variableWidget, false);
     mainWindow()->setViewAvailable(framestackWidget, false);
     mainWindow()->setViewAvailable(disassembleWidget, false);
     mainWindow()->setViewAvailable(gdbOutputWidget, false);
@@ -758,22 +748,41 @@ void DebuggerPart::slotStopDebugger()
     core()->running(this, false);
 }
 
+void DebuggerPart::slotShowView(bool show)
+{
+    const QWidget* s = static_cast<const QWidget*>(sender());
+    QWidget* ncs = const_cast<QWidget*>(s);
+    mainWindow()->setViewAvailable(ncs, show);
+    if (show)
+        mainWindow()->raiseView(ncs);
+}
+
 void DebuggerPart::slotDebuggerAbnormalExit()
 {
     mainWindow()->raiseView(gdbOutputWidget);
 
-    KMessageBox::error(
-        mainWindow()->main(), 
+    KMessageBox::warningYesNo(
+        mainWindow()->main(),
         i18n("<b>GDB exited abnormally</b>"
              "<p>This is likely a bug in GDB. "
              "Examine the gdb output window and then stop the debugger"),
-        i18n("GDB exited abnormally"));
+        i18n("GDB exited abnormally"), KStdGuiItem::ok(), KStdGuiItem::cont(), "gdb_error");
 
     // Note: we don't stop the debugger here, becuse that will hide gdb
     // window and prevent the user from finding the exact reason of the
     // problem.
 }
 
+void DebuggerPart::slotFileSaved()
+{
+    needRebuild_ = true;
+}
+
+void DebuggerPart::slotProjectCompiled()
+{
+    needRebuild_ = false;
+}
+
 void DebuggerPart::projectClosed()
 {
     slotStopDebugger();
@@ -781,10 +790,118 @@ void DebuggerPart::projectClosed()
 
 void DebuggerPart::slotRun()
 {
-    if( controller->stateIsOn( s_dbgNotStarted ) )
+    if( controller->stateIsOn( s_dbgNotStarted ) ||
+        controller->stateIsOn( s_appNotStarted ) )
+    {
+        if (running_ && controller->stateIsOn(s_dbgNotStarted))
+        {
+            // User has already run the debugger, but it's not running.
+            // Most likely, the debugger has crashed, and the debuggerpart
+            // was left in 'running' state so that the user can examine
+            // gdb output or something. But now, need to fully shut down
+            // previous debug session.
+            slotStopDebugger();
+        }
+
+        // We're either starting gdb for the first time,
+        // or starting the application under gdb. In both
+        // cases, might need to rebuild the application.
+
+        // Note that this logic somewhat duplicates the
+        // isDirty method present in a number of project plugins.
+        // But there, it's a private method we can't conveniently
+        // access. Besides, the custom makefiles project manager won't
+        // care about a file unless it's explicitly added, so it can
+        // miss dependencies.
+
+        needRebuild_ |= haveModifiedFiles();
+
+        bool rebuild = false;
+        if (needRebuild_ && project())
+        {
+            // We don't add "Don't ask again" checkbox to the
+            // message because it's not clear if one cooked
+            // decision will be right for all cases when we're starting
+            // debugging with modified code, and because it's not clear
+            // how user can reset this "don't ask again" setting.
+            int r = KMessageBox::questionYesNoCancel(
+                0,
+                "<b>" + i18n("Rebuild the project?") + "</b>" +
+                i18n("<p>The project is out of date. Rebuild it?"),
+                i18n("Rebuild the project?"));
+            if (r == KMessageBox::Cancel)
+            {
+                return;
+            }
+            if (r == KMessageBox::Yes)
+            {
+                rebuild = true;
+            }
+            else
+            {
+                // If the user said don't rebuild, try to avoid
+                // asking the same question again.
+                // Note that this only affects 'were any files changed'
+                // check, if a file is changed but not saved we'll
+                // still ask the user again. That's bad, but I don't know
+                // a better solution -- it's hard to check that
+                // the file has the same content as it had when the user
+                // last answered 'no, don't rebuild'.
+                needRebuild_ = false;
+            }
+
+            if (rebuild)
+            {
+                disconnect(SIGNAL(buildProject()));
+                // The KDevProject has no method to build the project,
+                // so try connecting to a slot has is present to all
+                // existing project managers.
+                // Note: this assumes that 'slotBuild' will save
+                // modified files.
+
+                if (connect(this, SIGNAL(buildProject()),
+                            project(), SLOT(slotBuild())))
+                {
+                    connect(project(), SIGNAL(projectCompiled()),
+                            this, SLOT(slotRun_part2()));
+
+                    emit buildProject();
+                    rebuild = true;
+                }
+            }
+        }
+        if (!rebuild)
+        {
+            slotRun_part2();
+        }
+        return;
+    }
+    else
+    {
+        // When continuing the program, don't try to rebuild -- user
+        // has explicitly asked to "continue".
+        mainWindow()->statusBar()->message(i18n("Continuing program"), 1000);
+    }
+    controller->slotRun();
+}
+
+void DebuggerPart::slotRun_part2()
+{
+    needRebuild_ = false;
+
+    disconnect(project(), SIGNAL(projectCompiled()),
+               this, SLOT(slotRun_part2()));
+
+    if (controller->stateIsOn( s_dbgNotStarted ))
     {
         mainWindow()->statusBar()->message(i18n("Debugging program"), 1000);
-        mainWindow()->raiseView(gdbOutputWidget);
+        if ( DomUtil::readBoolEntry( *projectDom(), "/kdevdebugger/general/raiseGDBOnStart", false ) )
+        {
+            mainWindow()->raiseView( gdbOutputWidget );
+        }else
+        {
+            mainWindow()->raiseView( framestackWidget );
+        }
         appFrontend()->clearView();
         startDebugger();
     }
@@ -803,20 +920,25 @@ void DebuggerPart::slotRun()
 
         appFrontend()->clearView();
     }
-    else
-    {
-        mainWindow()->statusBar()->message(i18n("Continuing program"), 1000);
-    }
+
     controller->slotRun();
 }
 
+
 void DebuggerPart::slotRestart()
 {
-    // We could have directly connect KAction to controller->slotRestart()
-    // but controller is created after actions, and I did not want to 
-    // create unconnected action and connect it later, as it would make
-    // it harder to understand the code.
-    controller->slotRestart();
+    // We implement restart as kill + slotRun, as opposed as plain "run"
+    // command because kill + slotRun allows any special logic in slotRun
+    // to apply for restart.
+    //
+    // That includes:
+    // - checking for out-of-date project
+    // - special setup for remote debugging.
+    //
+    // Had we used plain 'run' command, restart for remote debugging simply
+    // would not work.
+    controller->slotKill();
+    slotRun();
 }
 
 void DebuggerPart::slotExamineCore()
@@ -913,7 +1035,7 @@ void DebuggerPart::slotStepOver()
 
 void DebuggerPart::slotStepOverInstruction()
 {
-    controller->slotStepOver();
+    controller->slotStepOverIns();
 }
 
 
@@ -937,74 +1059,60 @@ void DebuggerPart::slotStepOut()
 
 void DebuggerPart::slotMemoryView()
 {
-    // Hmm, couldn't this be made non-modal?
-
-    MemoryViewDialog *dlg = new MemoryViewDialog();
-    connect( dlg,        SIGNAL(disassemble(const QString&, const QString&)),
-             controller, SLOT(slotDisassemble(const QString&, const QString&)));
-    connect( dlg,        SIGNAL(memoryDump(const QString&, const QString&)),
-             controller, SLOT(slotMemoryDump(const QString&, const QString&)));
-    connect( dlg,        SIGNAL(registers()),
-             controller, SLOT(slotRegisters()));
-    connect( dlg,        SIGNAL(libraries()),
-             controller, SLOT(slotLibraries()));
-
-    connect( controller, SIGNAL(rawGDBMemoryDump(char*)),
-             dlg,        SLOT(slotRawGDBMemoryView(char*)));
-    connect( controller, SIGNAL(rawGDBDisassemble(char*)),
-             dlg,        SLOT(slotRawGDBMemoryView(char*)));
-    connect( controller, SIGNAL(rawGDBRegisters(char*)),
-             dlg,        SLOT(slotRawGDBMemoryView(char*)));
-    connect( controller, SIGNAL(rawGDBLibraries(char*)),
-             dlg,        SLOT(slotRawGDBMemoryView(char*)));
-
-    dlg->exec();
-    delete dlg;
+    viewerWidget->slotAddMemoryView();
 }
 
-
 void DebuggerPart::slotRefreshBPState( const Breakpoint& BP)
 {
-    if (BP.type() == BP_TYPE_FilePos)
+    if (BP.hasFileAndLine())
     {
         const FilePosBreakpoint& bp = dynamic_cast<const FilePosBreakpoint&>(BP);
         if (bp.isActionDie())
+        {
             debugger()->setBreakpoint(bp.fileName(), bp.lineNum()-1, -1, true, false);
+        }
+        else if (bp.isActionClear())
+        {
+            // Do nothing. This is always a result of breakpoint deletion,
+            // either via click on gutter, or via breakpoints window.
+            // We should not add marker for a breakpoint that's being deleted,
+            // because if user removes marker, and we re-add it here until
+            // we see 'actionDie' this can confuse the code.
+            // And no need to clear the marker, since we'll soon see 'actionDie'
+            // and clear it for good.
+        }
         else
             debugger()->setBreakpoint(bp.fileName(), bp.lineNum()-1,
                                   1/*bp->id()*/, bp.isEnabled(), bp.isPending() );
     }
 }
 
-
 void DebuggerPart::slotStatus(const QString &msg, int state)
 {
-    QString stateIndicator;
+    QString stateIndicator, stateIndicatorFull;
 
     if (state & s_dbgNotStarted)
     {
         stateIndicator = " ";
-        mainWindow()->lowerView(variableWidget);
+        stateIndicatorFull = "Debugger not started";
+        stateChanged( QString("stopped") );
     }
-    else if (state & s_appBusy)
+    else if (state & s_dbgBusy)
     {
-        stateIndicator = "A";
-        debugger()->clearExecutionPoint();
+        stateIndicator = "R";
+        stateIndicatorFull = "Debugger is busy";
         stateChanged( QString("active") );
     }
     else if (state & s_programExited)
     {
         stateIndicator = "E";
+        stateIndicatorFull = "Application has exited";
         stateChanged( QString("stopped") );
-        KActionCollection *ac = actionCollection();
-        ac->action("debug_run")->setText( i18n("To start something","Start") );
-        ac->action("debug_run")->setToolTip( i18n("Restart the program in the debugger") );
-        ac->action("debug_run")->setWhatsThis( i18n("Restart in debugger\n\n"
-                                           "Restarts the program in the debugger") );
     }
     else
     {
         stateIndicator = "P";
+        stateIndicatorFull = "Application is paused";
         stateChanged( QString("paused") );
         // On the first stop, show the variables view.
         // We do it on first stop, and not at debugger start, because
@@ -1015,20 +1123,34 @@ void DebuggerPart::slotStatus(const QString &msg, int state)
         if (justRestarted_)
         {
             justRestarted_ = false;
+            mainWindow()->setViewAvailable(variableWidget, true);
             mainWindow()->raiseView(variableWidget);
         }
     }
 
+    if (state & s_appNotStarted)
+    {
+        KActionCollection *ac = actionCollection();
+        ac->action("debug_run")->setText( i18n("To start something","Start") );
+        ac->action("debug_run")->setToolTip( i18n("Restart the program in the debugger") );
+        ac->action("debug_run")->setWhatsThis( i18n("Restart in debugger\n\n"
+                                           "Restarts the program in the debugger") );
+    }
+
+
+    bool program_running = !(state & s_appNotStarted);
+    bool attached_or_core = (state & s_attached) || (state & s_core);
+
     // If program is started, enable the 'restart' comand.
     actionCollection()->action("debug_restart")->setEnabled(
-        !(state & s_programExited));
+        program_running && !attached_or_core);
 
 
     // As soon as debugger clears 's_appNotStarted' flag, we
-    // set 'justRestarted' variable. 
+    // set 'justRestarted' variable.
     // The other approach would be to set justRestarted in slotRun, slotCore
     // and slotAttach.
-    // Note that setting this var in startDebugger is not OK, because the 
+    // Note that setting this var in startDebugger is not OK, because the
     // initial state of debugger is exactly the same as state after pause,
     // so we'll always show varaibles view.
     if ((previousDebuggerState_ & s_appNotStarted) &&
@@ -1046,12 +1168,24 @@ void DebuggerPart::slotStatus(const QString &msg, int state)
     kdDebug(9012) << "   " << msg << endl;
 
     statusBarIndicator->setText(stateIndicator);
+    QToolTip::add(statusBarIndicator, stateIndicatorFull);
     if (!msg.isEmpty())
         mainWindow()->statusBar()->message(msg, 3000);
 
+
     previousDebuggerState_ = state;
 }
 
+void DebuggerPart::slotEvent(GDBController::event_t e)
+{
+    if (e == GDBController::program_running ||
+        e == GDBController::program_exited ||
+        e == GDBController::debugger_exited)
+    {
+        debugger()->clearExecutionPoint();
+    }
+}
+
 
 void DebuggerPart::slotShowStep(const QString &fileName, int lineNum)
 {
@@ -1060,6 +1194,10 @@ void DebuggerPart::slotShowStep(const QString &fileName, int lineNum)
         // Debugger counts lines from 1
         debugger()->gotoExecutionPoint(KURL( fileName ), lineNum-1);
     }
+    else
+    {
+        debugger()->clearExecutionPoint();
+    }
 }
 
 
@@ -1089,11 +1227,29 @@ void DebuggerPart::slotActivePartChanged( KParts::Part* part )
 void DebuggerPart::restorePartialProjectSession(const QDomElement* el)
 {
     gdbBreakpointWidget->restorePartialProjectSession(el);
+    gdbOutputWidget->restorePartialProjectSession(el);
 }
 
 void DebuggerPart::savePartialProjectSession(QDomElement* el)
 {
     gdbBreakpointWidget->savePartialProjectSession(el);
+    gdbOutputWidget->savePartialProjectSession(el);
+}
+
+bool DebuggerPart::haveModifiedFiles()
+{
+    bool have_modified = false;
+    KURL::List const& filelist = partController()->openURLs();
+    KURL::List::ConstIterator it = filelist.begin();
+    while ( it != filelist.end() )
+    {
+        if (partController()->documentState(*it) != Clean)
+            have_modified = true;
+
+        ++it;
+    }
+
+    return have_modified;
 }
 
 }
