@@ -2,6 +2,9 @@
 *   Copyright (C) 2003 by Roberto Raggi                                   *
 *   roberto@kdevelop.org                                                  *
 *                                                                         *
+*   Copyright (C) 2006 by Jens Dagerbo                                    *
+*   jens.dagerbo@swipnet.se                                               *
+*                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
@@ -9,25 +12,23 @@
 *                                                                         *
 ***************************************************************************/
 
-#include "settingsdialog.h"
 #include <klistbox.h>
 #include <kcombobox.h>
+#include <kapplication.h>
+#include <kstandarddirs.h>
+#include <kurlrequester.h>
+#include <kdebug.h>
+#include <klineedit.h>
+#include <kmessagebox.h>
+#include <klocale.h>
+
 #include <qfile.h>
 #include <qdir.h>
 #include <qregexp.h>
 #include <cstdlib>
 
-#if QT_VERSION < 0x030100 
-/* original source from qt-3.2.1/src/widgets/qlistbox.cpp
-QListBoxItem* QListBox::selectedItem() const
-{
-    if ( d->selectionMode != Single )
-	return 0;
-    if ( isSelected( currentItem() ) )
-	return  d->current;
-    return 0;
-}
-*/
+#include "settingsdialog.h"
+
 QListBoxItem* QListBox_selectedItem( QListBox* cpQListBox )
 {
 	if ( cpQListBox->selectionMode() != QListBox::Single )
@@ -36,57 +37,31 @@ QListBoxItem* QListBox_selectedItem( QListBox* cpQListBox )
 		return cpQListBox->item( cpQListBox->currentItem() );
 	return 0;
 }
-#endif
 
 SettingsDialog::SettingsDialog( QWidget* parent, const char* name, WFlags fl )
-		: SettingsDialogBase( parent, name, fl )
+: SettingsDialogBase( parent, name, fl )
 {
-	// Parse $KDEDIRS first, because it takes precedence over $KDEDIR
-	QStringList kdedirs = QStringList::split( ':', QFile::decodeName( ::getenv( "KDEDIRS" ) ) );
-	for ( QStringList::Iterator it = kdedirs.begin(); it != kdedirs.end(); ++it )
-		if ( !( *it ).isEmpty() )
-			* it += "/include";
-
-	QString kdedir = QFile::decodeName( ::getenv( "KDEDIR" ) );
-	if ( !kdedir.isEmpty() )
-		kdedirs.push_back( kdedir + "/include" );
-
-	kdedirs.push_back( "/usr/lib/kde/include" );
-	kdedirs.push_back( "/usr/local/kde/include" );
-	kdedirs.push_back( "/usr/local/include" );
-	kdedirs.push_back( "/usr/kde/include" );
-	kdedirs.push_back( "/usr/include/kde" );
-	kdedirs.push_back( "/usr/include" );
-	kdedirs.push_back( "/opt/kde3/include" );
-	kdedirs.push_back( "/opt/kde/include" );
-
-	for ( QStringList::ConstIterator it = kdedirs.begin(); it != kdedirs.end(); ++it )
+	KApplication::kApplication()->dirs()->addResourceType("include","include");
+	QStringList kdedirs=KApplication::kApplication()->dirs()->findDirs("include","");
+	for( QStringList::Iterator it=kdedirs.begin(); it!=kdedirs.end();    ++it )
 	{
 		QString kdedir = *it;
 		if ( !kdedir.isEmpty() && isValidKDELibsDir( kdedir ) )
 			if ( !kdeListBox->findItem( kdedir, ExactMatch ) )
 				kdeListBox->insertItem( kdedir );
 	}
+
+	kdeUrl->setMode( KFile::Directory | KFile::ExistingOnly | KFile::LocalOnly );
+
+	connect( addUrlButton, SIGNAL(clicked()), this, SLOT(addUrlButton_clicked()) );
 }
 
 SettingsDialog::~SettingsDialog()
 {}
 
-/*$SPECIALIZATION$*/
-void SettingsDialog::slotSelectionChanged( QListBoxItem* /* item */ )
+void SettingsDialog::slotSelectionChanged( QListBoxItem* )
 {
-#if QT_VERSION < 0x030100
-	if ( !QListBox_selectedItem( kdeListBox ) )
-	{
-#else
-	if ( !kdeListBox->selectedItem() )
-	{
-#endif
-		emit enabled( false );
-		return ;
-	}
-
-	emit enabled( true );
+	emit enabled( kdeListBox->selectedItem() != 0 );
 }
 
 bool SettingsDialog::isValidKDELibsDir( const QString & path ) const
@@ -99,6 +74,25 @@ QString SettingsDialog::kdeDir( ) const
 	return kdeListBox->currentText();
 }
 
+void SettingsDialog::addUrlButton_clicked()
+{
+	kdDebug(9000) << k_funcinfo << endl;
+
+	if ( isValidKDELibsDir( kdeUrl->url() ) )
+	{
+		kdeListBox->insertItem( kdeUrl->url() );
+		if ( QListBoxItem * item = kdeListBox->findItem( kdeUrl->url(), ExactMatch ) )
+		{
+			kdeListBox->setSelected( item, true );
+		}
+		kdeUrl->lineEdit()->clear();
+	}
+	else
+	{
+		KMessageBox::error( this, i18n("This doesn't appear to be a valid KDE include directory.\nPlease select a different directory."), i18n("Invalid Directory") );
+	}
+}
+
 
 #include "settingsdialog.moc"
 //kate: indent-mode csands; tab-width 4; space-indent off;
