@@ -26,6 +26,7 @@
 #include <kinstance.h>
 #include <kurl.h>
 #include <kaction.h>
+#include <kactionclasses.h>
 #include <kpopupmenu.h>
 #include <kconfig.h>
 
@@ -43,6 +44,7 @@
 #include <qheader.h>
 #include <qdir.h>
 #include <qstylesheet.h>
+#include <qstringlist.h>
 
 // namespace ?!?
 
@@ -90,19 +92,40 @@ ClassViewWidget::ClassViewWidget( ClassViewPart * part )
 				    m_part->actionCollection(), "classview_open_implementation" );
     m_actionOpenImplementation->setWhatsThis(i18n("<b>Open implementation</b><p>Opens a file where the selected item is defined (implemented) and jumps to the definition line."));
 
+	m_actionFollowEditor = new KToggleAction( i18n("Follow Editor"), KShortcut(), this, SLOT(slotFollowEditor()), m_part->actionCollection(), "classview_follow_editor" );
+
     KConfig* config = m_part->instance()->config();
     config->setGroup( "General" );
     setViewMode( config->readNumEntry( "ViewMode", KDevelop3ViewMode ) );
+    m_doFollowEditor = config->readBoolEntry( "FollowEditor", false );
 }
 
-ClassViewWidget::~ ClassViewWidget( )
+ClassViewWidget::~ClassViewWidget( )
 {
     KConfig* config = m_part->instance()->config();
     config->setGroup( "General" );
     config->writeEntry( "ViewMode", viewMode() );
+    config->writeEntry( "FollowEditor", m_doFollowEditor );
     config->sync();
 }
 
+template <class ModelType, class ListItemType>
+        static bool selectItemG( ItemDom item, const QMap<KSharedPtr<ModelType>, ListItemType*>& map )
+{
+    ModelType* c = dynamic_cast<ModelType*>( &(*item) );
+    
+    if( c ) {
+        KSharedPtr<ModelType> d( c );
+        typename QMap<KSharedPtr<ModelType>, ListItemType*>::ConstIterator it = map.find(d);
+        if( it != map.end() ) {
+            ( *it )->select();
+            return true;
+        }
+    }
+    return false;
+}
+
+
 void ClassViewWidget::slotExecuted( QListViewItem* item )
 {
     if( ClassViewItem* cbitem = dynamic_cast<ClassViewItem*>( item ) ){
@@ -120,11 +143,47 @@ void ClassViewWidget::clear( )
     m_projectItem = 0;
 }
 
+void restoreOpenNodes( QStringList & list, QListViewItem * item )
+{
+	if ( item && !list.isEmpty() )
+	{
+		if ( item->text( 0 ) == list.first() )
+		{
+ 			item->setOpen( true );
+			list.pop_front();
+			restoreOpenNodes( list, item->firstChild() );
+		}
+		else
+		{	
+			restoreOpenNodes( list, item->nextSibling() );
+		}
+	}
+}
+
+void storeOpenNodes( QValueList<QStringList> & openNodes, QStringList const & list, QListViewItem * item )
+{
+	if ( item )
+	{
+		if ( item->isOpen() )
+		{
+			QStringList mylist( list );
+			mylist << item->text( 0 );
+			openNodes << mylist;
+			storeOpenNodes( openNodes, mylist, item->firstChild() );
+		}
+		storeOpenNodes( openNodes, list, item->nextSibling() );
+	}
+}
+
 void ClassViewWidget::refresh()
 {
     if( !m_part->project() )
 	return;
 
+    QValueList<QStringList> openNodes;
+    storeOpenNodes( openNodes, QStringList(), firstChild() );
+    int scrollbarPos = verticalScrollBar()->value();
+
     clear();
     m_projectItem = new FolderBrowserItem( this, this, m_part->project()->projectName() );
     m_projectItem->setOpen( true );
@@ -137,6 +196,14 @@ void ClassViewWidget::refresh()
 	++it;
     }
 
+    QValueList<QStringList>::iterator itt = openNodes.begin();
+    while ( itt != openNodes.end() )
+    {
+        restoreOpenNodes ( *itt, firstChild() );
+        ++itt;
+    }
+    verticalScrollBar()->setValue( scrollbarPos );
+
     blockSignals( false );
 }
 
@@ -163,18 +230,17 @@ void ClassViewWidget::slotProjectClosed( )
 {
 }
 
+
 void ClassViewWidget::insertFile( const QString& fileName )
 {
     QString fn = URLUtil::canonicalPath( fileName );
-    if( !m_part->project()->isProjectFile(fn) )
-	return;
     //kdDebug() << "======================== insertFile(" << fn << ")" << endl;
 
     FileDom dom = m_part->codeModel()->fileByName( fn );
     if( !dom )
 	return;
 
-    fn = m_part->project()->relativeProjectFile( fn );
+    fn = URLUtil::relativePathToFile(m_part->project()->projectDirectory(), fn);
     QStringList path;
 
     switch( viewMode() )
@@ -209,15 +275,13 @@ void ClassViewWidget::insertFile( const QString& fileName )
 void ClassViewWidget::removeFile( const QString& fileName )
 {
     QString fn = URLUtil::canonicalPath( fileName );
-    if( !m_part->project()->isProjectFile(fn) )
-	return;
     //kdDebug() << "======================== removeFile(" << fn << ")" << endl;
 
     FileDom dom = m_part->codeModel()->fileByName( fn );
     if( !dom )
 	return;
 
-    fn = m_part->project()->relativeProjectFile( fn );
+    fn = URLUtil::relativePathToFile(m_part->project()->projectDirectory(), fn);
     QStringList path;
 
     switch( viewMode() )
@@ -255,6 +319,10 @@ void ClassViewWidget::contentsContextMenuEvent( QContextMenuEvent * ev )
 
     ClassViewItem* item = dynamic_cast<ClassViewItem*>( selectedItem() );
 
+    m_actionFollowEditor->plug( &menu );
+    m_actionFollowEditor->setChecked( m_doFollowEditor );
+    menu.insertSeparator();
+
     m_actionOpenDeclaration->setEnabled( item && item->hasDeclaration() );
     m_actionOpenImplementation->setEnabled( item && item->hasImplementation() );
 
@@ -299,8 +367,7 @@ void ClassViewWidget::contentsContextMenuEvent( QContextMenuEvent * ev )
     ev->consume();
 }
 
-void ClassViewWidget::setViewMode( int mode )
-{
+void ClassViewWidget::setViewMode( int mode ) {
     m_actionViewMode->setCurrentItem( mode );
 }
 
@@ -309,6 +376,86 @@ int ClassViewWidget::viewMode( ) const
      return m_actionViewMode->currentItem();
 }
 
+QString ClassViewItem::comment() {
+    return QString();
+}
+
+void ClassViewItem::select( ) {
+    setOpen(true);
+    listView()->setCurrentItem(this);
+    QListViewItem* c = firstChild();
+    int size = 0;
+    if(c != 0) {
+        while(c->nextSibling()) {
+            c = c->nextSibling();
+            size++;
+        }
+        listView()->ensureItemVisible(c);
+    }
+    listView()->ensureItemVisible(this);
+}
+
+
+bool ClassViewWidget::selectItem( ItemDom item ) {
+    if(!m_projectItem || !isVisible()) return false;
+    if(item->isFunctionDefinition() && dynamic_cast<FunctionDefinitionModel*>(&(*item)) != 0) {
+        FunctionList lst;
+        FileList fileList = m_part->codeModel()->fileList();
+        CodeModelUtils::findFunctionDeclarations( FindOp2(FunctionDefinitionDom( (FunctionDefinitionModel*)&(*item) )), fileList, lst );
+        if(lst.isEmpty()) return false;
+
+        item = *lst.begin();
+    }
+    
+    return m_projectItem->selectItem( item );
+}
+
+
+QString NamespaceDomBrowserItem::comment() {
+    return m_dom->comment();
+}
+
+bool FolderBrowserItem::selectItem(ItemDom item)
+{
+    if(item->kind() == CodeModelItem::Class) {
+        if( selectItemG ( item, m_classes ) ) return true;
+    }
+
+    if(item->kind() == CodeModelItem::Function) {
+        if( selectItemG ( item, m_functions ) ) return true;
+    }
+    
+    if(item->kind() == CodeModelItem::TypeAlias) {
+        if( selectItemG ( item, m_typeAliases ) ) return true;
+    }
+    
+    if(item->kind() == CodeModelItem::Variable) {
+        if( selectItemG ( item, m_variables ) ) return true;
+    }
+    
+    if(item->kind() == CodeModelItem::Namespace) {
+        ///searching for namespaces is currently not supported and not useful
+    }
+    
+    for( QMap<ClassDom, ClassDomBrowserItem*>::Iterator it = m_classes.begin(); it != m_classes.end(); ++it ) {
+        if( (*it)->selectItem(item) )
+            return true;
+    }
+    
+    for( QMap<QString, NamespaceDomBrowserItem*>::Iterator it = m_namespaces.begin(); it != m_namespaces.end(); ++it ) {
+        if( (*it)->selectItem(item) )
+            return true;
+    }
+
+    for( QMap<QString, FolderBrowserItem*>::Iterator it = m_folders.begin(); it != m_folders.end(); ++it ) {
+        if( (*it)->selectItem(item) ) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
 void FolderBrowserItem::processFile( FileDom file, QStringList& path, bool remove )
 {
     if( path.isEmpty() ){
@@ -394,7 +541,6 @@ void FolderBrowserItem::processNamespace( NamespaceDom ns, bool remove )
 	    listView()->removedText << ns->name();
 	}
 	delete( item );
-    emit m_widget->removedNamespace(ns->name());
 	item = 0;
     }
 }
@@ -496,6 +642,41 @@ void FolderBrowserItem::processVariable( VariableDom var, bool remove )
 }
 
 // ------------------------------------------------------------------------
+bool NamespaceDomBrowserItem::selectItem( ItemDom item) 
+{
+    if(item->kind() == CodeModelItem::Class) {
+        if( selectItemG ( item, m_classes ) ) return true;
+    }
+
+    if(item->kind() == CodeModelItem::Function) {
+        if( selectItemG ( item, m_functions ) ) return true;
+    }
+    
+    if(item->kind() == CodeModelItem::TypeAlias) {
+        if( selectItemG ( item, m_typeAliases ) ) return true;
+    }
+    
+    if(item->kind() == CodeModelItem::Variable) {
+        if( selectItemG ( item, m_variables ) ) return true;
+    }
+    
+    if(item->kind() == CodeModelItem::Namespace) {
+        ///currently not neccessary..
+    }
+    
+    for(QMap<ClassDom, ClassDomBrowserItem*>::Iterator it = m_classes.begin(); it != m_classes.end(); ++it) {
+        if( (*it)->selectItem(item) )
+            return true;
+    }
+    
+    for(QMap<QString, NamespaceDomBrowserItem*>::Iterator it = m_namespaces.begin(); it != m_namespaces.end(); ++it) {
+        if( (*it)->selectItem(item) )
+            return true;
+    }
+    return false;
+}
+
+
 void NamespaceDomBrowserItem::processNamespace( NamespaceDom ns, bool remove )
 {
     NamespaceDomBrowserItem* item = m_namespaces.contains( ns->name() ) ? m_namespaces[ ns->name() ] : 0;
@@ -633,6 +814,34 @@ void NamespaceDomBrowserItem::processVariable( VariableDom var, bool remove )
 }
 
 // ------------------------------------------------------------------------
+
+
+bool ClassDomBrowserItem::selectItem(ItemDom item) 
+{
+    if(item->kind() == CodeModelItem::Class) {
+        if( selectItemG ( item, m_classes ) ) return true;
+    }
+
+    if(item->kind() == CodeModelItem::Function) {
+        if( selectItemG ( item, m_functions ) ) return true;
+    }
+    
+    if(item->kind() == CodeModelItem::TypeAlias) {
+        if( selectItemG ( item, m_typeAliases ) ) return true;
+    }
+    
+    if(item->kind() == CodeModelItem::Variable) {
+        if( selectItemG ( item, m_variables ) ) return true;
+    }
+    
+    for(QMap<ClassDom, ClassDomBrowserItem*>::Iterator it = m_classes.begin(); it != m_classes.end(); ++it) {
+        if( (*it)->selectItem(item) )
+            return true;
+    }
+    return false;
+}
+
+
 void ClassDomBrowserItem::processClass( ClassDom klass, bool remove )
 {
     ClassDomBrowserItem* item = m_classes.contains( klass ) ? m_classes[ klass ] : 0;
@@ -790,7 +999,8 @@ void FunctionDomBrowserItem::setup( )
     setPixmap( 0, UserIcon(iconName, KIcon::DefaultState, listView()->m_part->instance()) );
 
     QString txt = listView()->m_part->languageSupport()->formatModelItem(m_dom.data(), true);
-    setText( 0, txt );
+    
+    item() = highlightFunctionName(txt, 1, m_styles);
 }
 
 void FunctionDomBrowserItem::openDeclaration()
@@ -998,7 +1208,7 @@ void ClassViewWidget::maybeTip( QPoint const & p )
 			ArgumentList::ConstIterator it( list.begin() );
 			while ( it != list.end() )
 			{
-				arguments << (*it)->type();
+				arguments << ((*it)->type() + " " + (*it)->name());
 				++it;
 			}
 			
@@ -1039,7 +1249,14 @@ void ClassViewWidget::maybeTip( QPoint const & p )
 			tooltip = QString( "[Type] " ) + titem->dom()->type() + " " + titem->dom()->name();
 		}
 	}
+        
+        const int maxCommentSize = 300;
 	
+        if( !item->comment().isEmpty() ) {
+            tooltip += "\n";
+            tooltip += item->comment().length() > maxCommentSize ? item->comment().left( maxCommentSize ) + " [...]" : item->comment();
+        }
+        
 	kdDebug(0) << tooltip << endl;
 	
 	QRect r = itemRect( item );
@@ -1064,6 +1281,15 @@ void ClassViewWidget::slotCreateAccessMethods( )
 	}
 }
 
+void ClassViewWidget::slotFollowEditor()
+{
+	m_doFollowEditor = m_actionFollowEditor->isChecked();
+}
+
+bool ClassViewWidget::doFollowEditor()
+{
+	return m_doFollowEditor;
+}
+
 
 #include "classviewwidget.moc"
-
