@@ -43,33 +43,48 @@ CompileErrorFilter::ErrorFormat* CompileErrorFilter::errorFormats()
 	/// @todo could get these from emacs compile.el
 	static ErrorFormat formats[] = {
 		// GCC - another case, eg. for #include "pixmap.xpm" which does not exists
-		ErrorFormat( "([^: \t]+):([0-9]+):(?:[0-9]+):([^0-9]+)", 1, 2, 3 ),
+		ErrorFormat( "^([^:\t]{1,1024}):([0-9]{1,10}):(?:[0-9]+):([^0-9]+)", 1, 2, 3 ),
 		// GCC
-		ErrorFormat( "([^: \t]+):([0-9]+):([^0-9]+)", 1, 2, 3 ),
+		ErrorFormat( "^([^:\t]{1,1024}):([0-9]{1,10}):([^0-9]+)", 1, 2, 3 ),
 		// ICC
-		ErrorFormat( "([^: \\t]+)\\(([0-9]+)\\):([^0-9]+)", 1, 2, 3, "intel" ),
+		ErrorFormat( "^([^:\\t]{1,1024})\\(([0-9]{1,10})\\):([^0-9]+)", 1, 2, 3, "intel" ),
 		//libtool link
 		ErrorFormat( "(libtool):( link):( warning): ", 0, 0, 0 ),
+		// ld
+		ErrorFormat( "undefined reference", 0, 0, 0 ),
+		ErrorFormat( "undefined symbol", 0, 0, 0 ),
+		ErrorFormat( "ld: cannot find", 0, 0, 0 ),
+		ErrorFormat( "No such file", 0, 0, 0 ),
+		// make
+ 		ErrorFormat( "No rule to make target", 0, 0, 0 ),
 		// Fortran
 		ErrorFormat( "\"(.*)\", line ([0-9]+):(.*)", 1, 2, 3 ),
 		// Jade
 		ErrorFormat( "[a-zA-Z]+:([^: \t]+):([0-9]+):[0-9]+:[a-zA-Z]:(.*)", 1, 2, 3 ),
+		// ifort
+		ErrorFormat( "fortcom: Error: (.*), line ([0-9]+):(.*)", 1, 2, 3, "intel" ),
+		// PGI
+		ErrorFormat( "PGF9(.*)-(.*)-(.*)-(.*) \\((.*): ([0-9]+)\\)", 5, 6, 4, "pgi" ),
+		// PGI (2)
+		ErrorFormat( "PGF9(.*)-(.*)-(.*)-Symbol, (.*) \\((.*)\\)", 5, 5, 4, "pgi" ),
 
 		ErrorFormat( 0, 0, 0, 0 ) // this one last
 	};
 
 	return formats;
+
 }
 
 void CompileErrorFilter::processLine( const QString& line )
 {
-       	bool hasmatch = false;
+	bool hasmatch = false;
 	QString file;
 	int lineNum = 0;
 	QString text;
 	QString compiler;
 	int i = 0;
 	bool isWarning = false;
+	bool isInstantiationInfo = false;
 	ErrorFormat* const errFormats = errorFormats();
 	ErrorFormat* format = &errFormats[i];
 	while( !format->expression.isEmpty() )
@@ -77,13 +92,17 @@ void CompileErrorFilter::processLine( const QString& line )
 		QRegExp& regExp = format->expression;
 
 		if ( regExp.search( line ) != -1 ) {
-	                hasmatch = true;
-        	        file    = regExp.cap( format->fileGroup );
-	                lineNum = regExp.cap( format->lineGroup ).toInt() - 1;
-	                text    = regExp.cap( format->textGroup );
+			hasmatch = true;
+			file    = regExp.cap( format->fileGroup );
+			lineNum = regExp.cap( format->lineGroup ).toInt() - 1;
+			text    = regExp.cap( format->textGroup );
 			compiler = format->compiler;
-			if (regExp.cap(3).contains("warning", false))
+			QString cap = regExp.cap(3);
+			if (cap.contains("warning:", false) || cap.contains("Warnung:", false))
 				isWarning = true;
+			if (regExp.cap(3).contains("instantiated from", false)) {
+				isInstantiationInfo = true;
+			}
 			break;
 		}
 
@@ -100,7 +119,7 @@ void CompileErrorFilter::processLine( const QString& line )
 
 	if ( hasmatch )
 	{
-		emit item( new ErrorItem( file, lineNum, text, line, isWarning, compiler ) );
+		emit item( new ErrorItem( file, lineNum, text, line, isWarning, isInstantiationInfo, compiler ) );
 	}
 	else
 	{
