@@ -162,17 +162,13 @@ MakeWidget::MakeWidget(MakeViewPart *part)
 {
 	updateSettingsFromConfig();
 
-/*	if ( m_compilerOutputLevel == eFull )
-	{
-		setTextFormat( Qt::LogText );
-	}
-	else
-	{*/
-		setTextFormat( Qt::RichText );
-//	}
+	setTextFormat( Qt::RichText );
 	
 	if ( m_bLineWrapping )
 		setWordWrap(WidgetWidth);
+	else
+		setWordWrap(NoWrap);
+
 	setWrapPolicy(Anywhere);
 	setReadOnly(true);
 	setMimeSourceFactory(new QMimeSourceFactory);
@@ -268,6 +264,7 @@ void MakeWidget::startNextJob()
 
 	it = dirList.begin();
 	QString dir = *it;
+	m_lastBuildDir = dir;
 	dirList.remove(it);
 
 	clear(); // clear the widget
@@ -471,7 +468,8 @@ QString MakeWidget::guessFileName( const QString& fName, int parag ) const
         // now it gets tricky - no directory navigation messages,
         // no absolute path - let's guess.
         name = fName;
-        if ( !checkFileExists( m_part->project()->projectDirectory() + "/" + fName, name ) &&
+        if ( !checkFileExists( m_lastBuildDir + "/" + fName, name) &&
+            !checkFileExists( m_part->project()->projectDirectory() + "/" + fName, name ) &&
             !checkFileExists( m_part->project()->projectDirectory() + "/" + m_part->project()->activeDirectory() + "/" + fName, name ) &&
             !checkFileExists( m_part->project()->buildDirectory() + "/" + fName, name ) )
             specialCheck(fName, name);
@@ -506,16 +504,8 @@ void MakeWidget::searchItem(int parag)
 	{
 		// open the file
 		kdDebug(9004) << "Opening file: " << guessFileName(item->fileName, parag) << endl;
-		if (item->m_cursor) {
-			uint line, col;
-			item->m_cursor->position(&line, &col);
-			kdDebug() << "Cursor new position: " << col << endl;
-			m_part->partController()->editDocument(KURL( guessFileName(item->fileName, parag) ), line, col);
-		} else {
-			m_part->partController()->editDocument(KURL( guessFileName(item->fileName, parag) ), item->lineNum);
-		}
+		m_part->partController()->editDocument(KURL( guessFileName(item->fileName, parag) ), item->lineNum);
 		m_part->mainWindow()->statusBar()->message( item->m_error, 10000 );
-		m_part->mainWindow()->lowerView(this);
         m_lastErrorSelected = parag;
 	}
 }
@@ -644,10 +634,6 @@ bool MakeWidget::appendToLastLine( const QString& text )
 
 void MakeWidget::insertItem( MakeItem* new_item )
 {
-	ErrorItem* e = dynamic_cast<ErrorItem*>(new_item);
-	if (e)
-		createCursor(e, 0L);
-
   displayPendingItem();
   m_pendingItem = new_item;
 
@@ -662,69 +648,6 @@ void MakeWidget::insertItem( MakeItem* new_item )
   }
 }
 
-void MakeWidget::slotDocumentOpened( const KURL & filename )
-{
-	KParts::Part* part = m_part->partController()->partForURL(filename);
-	KTextEditor::Document* doc = dynamic_cast<KTextEditor::Document*>(part);
-
-	if (!doc) {
-		kdWarning() << k_funcinfo << "Couldn't find the document that was just opened." << endl;
-		return;
-	}
-
-	connect(part, SIGNAL(destroyed(QObject*)), this, SLOT(slotDocumentClosed(QObject*)) );
-
-	for (QValueVector<MakeItem*>::iterator it = m_items.begin(); it != m_items.end(); ++it) {
-		ErrorItem* e = dynamic_cast<ErrorItem*>(*it);
-
-		if (!e || e->m_cursor) continue;
-
-		if (filename.path().endsWith(e->fileName))
-			createCursor(e, doc);
-	}
-}
-
-void MakeWidget::createCursor(ErrorItem* e, KTextEditor::Document* doc)
-{
-	// Disabled for now - comment out to test. You need an up-to-date cvs head katepart to avoid mem leaks
-    return;
-
-	// try to get a KTextEditor::Cursor, so that we can retain position in
-	// a document even when it is edited
-	if (!doc)
-		doc = dynamic_cast<KTextEditor::Document*>(m_part->partController()->partForURL(KURL( guessFileName(e->fileName, m_paragraphs + 1 ))));
-
-	if (doc) {
-		KTextEditor::EditInterface* edit = dynamic_cast<KTextEditor::EditInterface*>(doc);
-		KTextEditor::CursorInterface* cursor = dynamic_cast<KTextEditor::CursorInterface*>(doc);
-		if (cursor) {
-			e->m_cursor = cursor->createCursor();
-			uint col = 0;
-			static QRegExp startOfText("[\\S]");
-			if (edit) {
-				int newcol = edit->textLine(e->lineNum).find(startOfText);
-				if (newcol != -1) col = uint(newcol);
-			}
-			e->m_cursor->setPosition(e->lineNum, col);
-			e->m_doc = doc;
-		}
-	}
-}
-
-void MakeWidget::slotDocumentClosed(QObject* doc)
-{
-	KTextEditor::Document* document = static_cast<KTextEditor::Document*>(doc);
-
-	for (QValueVector<MakeItem*>::iterator it = m_items.begin(); it != m_items.end(); ++it) {
-		ErrorItem* e = dynamic_cast<ErrorItem*>(*it);
-
-		if (!e || e->m_doc != document) continue;
-
-		e->m_cursor = 0L;
-		e->m_doc = 0L;
-	}
-}
-
 bool MakeWidget::brightBg()
 {
 	int h,s,v;
@@ -819,8 +742,10 @@ void MakeWidget::toggleShowDirNavigMessages()
 void MakeWidget::updateSettingsFromConfig()
 {
 	KConfig *pConfig = kapp->config();
+	pConfig->setGroup("General Options");
+	QFont outputFont = pConfig->readFontEntry("OutputViewFont");
+	setFont(outputFont);
 	pConfig->setGroup("MakeOutputView");
-	setFont(pConfig->readFontEntry("Messages Font"));
 	m_bLineWrapping = pConfig->readBoolEntry("LineWrapping", true);
 	m_compilerOutputLevel = (EOutputLevel) pConfig->readNumEntry("CompilerOutputLevel", (int) eShort);
 	DirectoryItem::setShowDirectoryMessages( pConfig->readBoolEntry("ShowDirNavigMsg", false) );
@@ -835,6 +760,8 @@ bool MakeWidget::scanErrorForward( int parag )
 		ErrorItem* item = dynamic_cast<ErrorItem*>( m_paragraphToItem[it] );
 		if ( !item )
 			continue;
+		if( item->m_isWarning )
+			continue;
 		parag = it;
 		document()->removeSelection(0);
 		setSelection(parag, 0, parag+1, 0, 0);
@@ -853,6 +780,8 @@ bool MakeWidget::scanErrorBackward( int parag )
 		ErrorItem* item = dynamic_cast<ErrorItem*>( m_paragraphToItem[it] );
 		if ( !item )
 			continue;
+		if( item->m_isWarning )
+			continue;
 		parag = it;
 		document()->removeSelection(0);
 		setSelection(parag, 0, parag+1, 0, 0);
