@@ -23,6 +23,7 @@
  
 #include <klistbox.h>
 #include <klineedit.h>
+#include <kdebug.h>
 
 #include "quickopendialog.h"
 #include "quickopen_part.h"
@@ -40,7 +41,12 @@ QuickOpenDialog::~QuickOpenDialog()
 
 void QuickOpenDialog::slotTextChanged(const QString & text)
 {
-    m_typeTimeout.start( 300, true );
+    m_typeTimeout.start( 100, true );
+}
+
+void QuickOpenDialog::maybeUpdateSelection() {
+    if( m_typeTimeout.isActive() )
+        slotTextChangedDelayed();
 }
 
 void QuickOpenDialog::slotTextChangedDelayed()
@@ -65,7 +71,7 @@ bool QuickOpenDialog::eventFilter( QObject * watched, QEvent * e )
             {
                 itemList->setCurrentItem(i);
                 nameEdit->blockSignals(true);
-                nameEdit->setText(itemList->currentText());
+                itemSelectionChanged();
                 nameEdit->blockSignals(false);
             }
             return true;
@@ -76,7 +82,7 @@ bool QuickOpenDialog::eventFilter( QObject * watched, QEvent * e )
             {
                 itemList->setCurrentItem(i);
                 nameEdit->blockSignals(true);
-                nameEdit->setText(itemList->currentText());
+                itemSelectionChanged();
                 nameEdit->blockSignals(false);
             }
             return true;
@@ -84,7 +90,7 @@ bool QuickOpenDialog::eventFilter( QObject * watched, QEvent * e )
         {
             QApplication::sendEvent(itemList, e);
             nameEdit->blockSignals(true);
-            nameEdit->setText(itemList->currentText());
+            itemSelectionChanged();
             nameEdit->blockSignals(false);
         }
     }
@@ -108,10 +114,7 @@ QStringList QuickOpenDialog::wildCardCompletion(const QString & text)
     {
         if ( (*it).find( re ) != -1 )
         {
-            if ( !matches.contains( *it ) )
-            {
-                matches << *it;
-            }
+            matches << *it;
         }
         ++it;
     }
@@ -119,5 +122,29 @@ QStringList QuickOpenDialog::wildCardCompletion(const QString & text)
     return matches;
 }
 
+void QuickOpenDialog::QStringList_unique( QStringList & list )
+{
+    if ( list.size() < 2 ) return;
+
+    list.sort();
+
+    QStringList::iterator it = list.begin();
+    QStringList::iterator it2 = it;
+    while ( it2 != list.end() )
+    {
+        ++it2;
+        while ( it2 != list.end() && *it2 == *it )
+        {
+            it2 = list.remove( it2 );
+        }
+        it = it2;
+    }
+}
+
+void QuickOpenDialog::itemSelectionChanged() {
+  nameEdit->setText(itemList->currentText());
+}
+
 #include "quickopendialog.moc"
 
+// kate: space-indent on; indent-width 4; tab-width 4; show-tabs on;
